//
// C++ Implementation: kpgdatatable
//
// Description: 
//
//
// Author: Lumir Vanek <lvanek@users.sourceforge.net>, (C) 2006
//
// Copyright: See COPYING file that comes with this distribution
//
//
#include "kpgdatatable.h"

// include files for Qt
#include <qtable.h>
#include <qlayout.h>
#include <qtextcodec.h>

// include files for KDE
#include <klocale.h>
#include <kmessagebox.h>
#include <kcursor.h>

// application specific includes
#include "../DbObjects/kpgconnection.h"
#include "../DbObjects/kpgdatabase.h"
#include "../DbObjects/kpgtable.h"


KPGDataTable::KPGDataTable(QWidget *parent, 
	KPGConnection *pConnection, 
	const QString & strNamespace,
	KPGTable *pTable)
 : QWidget(parent, "Data table")
{
	m_pConnection = pConnection;
	m_strNamespace = strNamespace;
	m_pTable = pTable;
	
	QHBoxLayout *pLayout = new QHBoxLayout(this, 0, -1, "Layout1"); 
  	m_pDataTable = new QTable( this, "DataTable" );
  	pLayout->addWidget( m_pDataTable, 0, 0 );
  	
  	m_pDataTable->setSorting( false );
  	m_pDataTable->setSelectionMode( QTable::SingleRow );
  	
  	m_ConnectionInThread.setEventReceiver(this);
  	
  	m_nTotalRows = 0;
	m_nTotalCols = 0;
}


KPGDataTable::~KPGDataTable()
{
}

void KPGDataTable::runQuery(KPGDatabase *pDatabase)
{
	// clear previous result
	m_pDataTable->setNumRows(0);
	m_pDataTable->setNumCols(0);
	m_pqxxResult.clear(); // free memory
		
	QString strConectionString( );
	
	setCursor(KCursor::waitCursor());
		
	m_ConnectionInThread.disconnectFromServer(); // close connection if any
	m_ConnectionInThread.connectToServer(pDatabase->connection()->options());
	m_ConnectionInThread.setSQL("SELECT * FROM" + m_strNamespace + "." + m_pTable->quotedName(), KPGConnection::eTransNormal, pDatabase->encoderName());
	m_ConnectionInThread.start();
	//emit sigQueryResultStateChanged(this);
	//emit sigEnableDbFetchActions(false, false);
}

void KPGDataTable::customEvent( QCustomEvent *pEvent)
{
	PKGQueryResultEvent *pQueryResultEvent = static_cast <PKGQueryResultEvent *> (pEvent);
	
	// display result
	if(pQueryResultEvent->result().size() > 0)
	{
		displayResult(pQueryResultEvent->result(), pQueryResultEvent->encoderName()); 
	}
	
	
	setCursor(KCursor::arrowCursor());
	
	m_ConnectionInThread.wait();
	//emit sigQueryResultStateChanged(this);
}

// Display SQL result
void KPGDataTable::displayResult(pqxx::result pqxxResult, const QString& strEncoderName)
{
	m_pqxxResult = pqxxResult;
	
	// Fill output area with query result
	m_nTotalRows = pqxxResult.size();
	m_nTotalCols = pqxxResult.columns();
		
	m_pDataTable->setNumRows(m_nTotalRows);
	m_pDataTable->setNumCols(m_nTotalCols);
	
	// field names
	QHeader* hHeader = m_pDataTable->horizontalHeader();
	
	for(unsigned int nCol = 0; nCol < m_nTotalCols; nCol++)
	{
		hHeader->setLabel(nCol, pqxxResult.column_name(nCol));
	}
	
	//m_strEncoderName = strEncoderName;
	QTextCodec *pTextCodec = QTextCodec::codecForName(strEncoderName);
	if(!pTextCodec)
		pTextCodec = QTextCodec::codecForLocale(); // hope that correct codec is returned
	
	// data
	for(unsigned int nRow = 0; nRow < m_nTotalRows; nRow++)
	{
		for(unsigned int nCol = 0; nCol < m_nTotalCols; nCol++)
		{
			QString strValue(pTextCodec->toUnicode(pqxxResult[nRow][nCol].c_str()));
			m_pDataTable->setText(nRow, nCol, strValue);
		}
	}
	
	for(unsigned int nCol = 0; nCol < m_nTotalCols; nCol++)
	{
		m_pDataTable->adjustColumn(nCol);
	}
	
	pqxxResult.clear(); // free memory
}

void KPGDataTable::stopQuery()
{
	if(m_ConnectionInThread.running ())
	{
		m_ConnectionInThread.terminate();
		m_ConnectionInThread.wait();
		m_ConnectionInThread.leaveConnection();
		
		
		setCursor(KCursor::arrowCursor());
		//emit sigQueryResultStateChanged(this);
	} 
}

