//
// C++ Interface: kpgdatatable
//
// Description: 
//
//
// Author: Lumir Vanek <lvanek@users.sourceforge.net>, (C) 2006
//
// Copyright: See COPYING file that comes with this distribution
//
//
#ifndef KPGDATATABLE_H
#define KPGDATATABLE_H

#include <qwidget.h>

#include "../kpgconnectioninthread.h"

class QTable;
class KPGConnection;
class KPGTable;
class KPGDatabase;

/**
Editable datatable to edit database table or view

	@author Lumir Vanek <lvanek@users.sourceforge.net>
*/
class KPGDataTable : public QWidget
{
	Q_OBJECT
public:
    KPGDataTable(QWidget *parent, KPGConnection *, const QString&, KPGTable *);
    ~KPGDataTable();

	// Run SQL query
	void runQuery(KPGDatabase *);
	
	// Terminate thread that run SQL query
	void stopQuery();
	
	// Returns TRUE if the connection thread is running; otherwise returns FALSE.
	bool running() const { return  m_ConnectionInThread.running(); }
	
protected:

    // Receive event from KPGConnectionInThread
    virtual void customEvent( QCustomEvent * );

    // Display SQL result
    void displayResult(pqxx::result, const QString &);  
	
protected:
	// GUI widget for editing table data
    QTable* m_pDataTable;
    
    // Connection to database
    KPGConnection* m_pConnection;
    
    // Database table to edit its data
    KPGTable *m_pTable;
    
    // Table namespace
    QString m_strNamespace;
    
    // Thread that run queries on background
	KPGConnectionInThread m_ConnectionInThread;
	
    // Resultset to display and edit
	pqxx::result m_pqxxResult;
	
	unsigned int m_nTotalRows; // Number of rows in result
	unsigned int m_nTotalCols; // Number oc columns in result
};

#endif
