/***************************************************************************
                          kpgconnection.h  -  description
                             -------------------
    begin                : � led 13 2004
    copyright            : (C) 2004 by Lumir Vanek
    email                : lvanek@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef KPGCONNECTION_H
#define KPGCONNECTION_H

// include files for libpgxx 
#include <pqxx/connection.h>

using namespace pqxx;

class QString;
class QStringList;

// Define a pqxx::noticer to process warnings generated by the database 
struct KPGNoticer : noticer
{
  virtual void operator()(const char Msg[]) throw ();
	QStringList *m_pListOfNotifications;
};


/**
Item that represent connection to PostgreSQL server

@author Lumir Vanek
*/

class KPGConnection : public connection
{
public: 
	KPGConnection(const QString);
	virtual ~KPGConnection() throw ();
	
  void setPassword(const QString & strPassword) { m_strPassword = strPassword; }
  const QString password() const { return m_strPassword; }
  
	// Return encoder name for QTextCodec regards to DB encoding
	const QString encoderName(const QString &) const;
	
	// Return list of notifications
	const QStringList & notifications() const { return m_listOfNotifications; }
	void clearNotifications();
	
	enum eTransType { eTransNormal, eTransNone, eTransRobust };
	
	pqxx::result runQuery(const QString &, eTransType = eTransNormal);
	pqxx::result runQuery(const QString &, const QString &, eTransType = eTransNormal);
  
  void setTrace();
  
private:

    /*
    It's here to resolve problem:
    
    [libpqxx-general] undefined reference to pqxx::connection::do_startconnect()' with libpqxx-2.5.5
    
    http://gborg.postgresql.org/pipermail/libpqxx-general/2005-November/001043.html
    */
    //virtual void startconnect() { do_startconnect2(); }
    //void do_startconnect2();


protected:
    QStringList m_listOfNotifications;
    FILE *m_pFileTrace;
    QString m_strPassword; // remember password, that is used for connection
};

#endif
