//
// C++ Interface: %{MODULE}
//
// Description: 
//
//
// Author: %{AUTHOR} <%{EMAIL}>, (C) %{YEAR}
//
// Copyright: See COPYING file that comes with this distribution
//
//
#ifndef KPGCONNECTIONINTHREAD_H
#define KPGCONNECTIONINTHREAD_H

// include files for Qt
#include <qstringlist.h>
#include <qthread.h>
#include <qevent.h>

#include "DbObjects/kpgconnection.h"

/**
Event, that contain SQL operation result

@author Lumir Vanek
*/

class PKGQueryResultEvent : public QCustomEvent
{
public:
    PKGQueryResultEvent(const QString &strEncoderName, KPGConnection * pConnection)
      : QCustomEvent( 65432 ) 
    {
      m_strEncoderName = strEncoderName;
      m_pConnection = pConnection;
      m_iExecutionMiliseconds = 0;
    }
     
    pqxx::result & result() { return m_pqxxResult; }
    void setError(const QString & strError) { m_strError = strError; }
    const QString & error() const { return m_strError; }
    const QString & encoderName() const { return m_strEncoderName; }
    KPGConnection * connection() { return m_pConnection; }
    
    void setExecutionMiliseconds(int iExecutionMiliseconds) { m_iExecutionMiliseconds = iExecutionMiliseconds; }
    int executionMiliseconds() const { return m_iExecutionMiliseconds; }

private:
    pqxx::result m_pqxxResult;
    QString m_strError;
    QString m_strEncoderName;
    KPGConnection *m_pConnection;
    int m_iExecutionMiliseconds;
};

/**
Thread, that run connection to PostgreSQL server

@author Lumir Vanek
*/

class KPGConnectionInThread : public QThread
{
public:
    KPGConnectionInThread();
    ~KPGConnectionInThread();
    
    bool connectToServer(const QString &);
    void disconnectFromServer();
    
    // Call it when connection thread is terminated
    void leaveConnection();

    void setSQL(const QString &strSQL, KPGConnection::eTransType _eTransType, const QString &strEncoderName) 
    { m_strSQL = strSQL; m_eTransType = _eTransType; m_strEncoderName = strEncoderName; }
    void setEventReceiver(QObject * pEventReceiver) { m_pEventReceiver = pEventReceiver; }
        
protected:
    virtual void run();
    
protected:
  KPGConnection *m_pConnection;
  QString m_strSQL; // SQL that is used for perform action
  QString m_strEncoderName;
  KPGConnection::eTransType m_eTransType;
  QObject *m_pEventReceiver; // event receiver
};

#endif
