/***************************************************************************
                          kpgsqlquerresultwindow.cpp  -  description
                             -------------------
    begin                : So led 17 2004
    copyright            : (C) 2004 by Lumir Vanek
    email                : lvanek@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "kpgsqlquerresultwindow.h"

// include files for Qt
//#include <qmultilineedit.h>
#include <qtable.h>
#include <qwidgetstack.h> 
#include <qtextcodec.h>

// include files for KDE
#include <ktextedit.h>
#include <kcombobox.h>
#include <kcursor.h>
#include <kmessagebox.h>
#include <kdebug.h>

// application specific includes
#include "kpgsyntaxhighlighter.h"
#include "kpogreview.h"
#include "kpgconfiguration.h"
#include "kpgsqleditorsettings.h"
#include "DbObjects/kpgconnection.h"
#include "DbObjects/kpgserver.h"
#include "DbObjects/kpgdatabase.h"

#define NUM_OF_RECORDS_PER_PAGE 500 // How many records are loaded to result table 


KPGSqlQueryResultWindow::KPGSqlQueryResultWindow(QWidget *parent, const char *name )
: KPGSqlQueryResultWindowBase(parent, name)
{
	m_pSyntaxHighlighter = new KPGSyntaxHighlighter(m_pTextEditQuery);
	m_pTextEditQuery->setText("SELECT * FROM ;");
	m_pTextEditQuery->setSelection(14 , 0, 14, 0),

 	// Apply current configuration
	slotSqlEditorSettingsChanged();
 	// and make sure to be informed about its changes.
	connect( KPoGreView::configuration()->sqleditor(), SIGNAL(sigChanged()), this, SLOT(slotSqlEditorSettingsChanged()) );
	
	connect(m_pTextEditQuery, SIGNAL(textChanged()), this, SLOT(slotTextEditQueryTextChanged()));
	connect(m_pTextEditQuery, SIGNAL(copyAvailable(bool)), this, SLOT(slotTextEditCopyAvailable(bool)));
	connect(m_pTextEditQuery, SIGNAL(undoAvailable(bool)), this, SLOT(slotTextEditUndoAvailable(bool)));
	connect(m_pTextEditQuery, SIGNAL(redoAvailable(bool)), this, SLOT(slotTextEditRedoAvailable(bool)));
	
	m_bQueryTextChanged = false;
  
	m_ConnectionInThread.setEventReceiver(this);
	
	m_nTotalRows = 0;
	m_nTotalCols = 0;
	m_nFetchedRows = 0;
}

KPGSqlQueryResultWindow::~KPGSqlQueryResultWindow()
{
  	delete m_pSyntaxHighlighter;
}

void KPGSqlQueryResultWindow::slotSqlEditorSettingsChanged()
{
	m_pSyntaxHighlighter->setColorDefaultText( KPoGreView::configuration()->sqleditor()->colorDefaultText() );
	m_pSyntaxHighlighter->setColorKeyWords( KPoGreView::configuration()->sqleditor()->colorKeyWords() );
	m_pSyntaxHighlighter->setColorDataTypes( KPoGreView::configuration()->sqleditor()->colorDataTypes() );
	m_pSyntaxHighlighter->setColorOperators( KPoGreView::configuration()->sqleditor()->colorOperators() );
	m_pSyntaxHighlighter->setColorQuotedStrings( KPoGreView::configuration()->sqleditor()->colorQuotedStrings() );
	m_pSyntaxHighlighter->setColorNumbers( KPoGreView::configuration()->sqleditor()->colorNumbers() );
    m_pSyntaxHighlighter->setColorComments( KPoGreView::configuration()->sqleditor()->colorComments() );
	
    m_pTextEditQuery->setFont( KPoGreView::configuration()->sqleditor()->font() );
    
	m_pSyntaxHighlighter->rehighlight();
}

void KPGSqlQueryResultWindow::slotServerChanged(const QString& strServer)
{
	emit sigRequestDatabasesList(this, strServer);    
}

void KPGSqlQueryResultWindow::slotTextEditQueryTextChanged()
{
	if(!m_bQueryTextChanged)
	{
		m_bQueryTextChanged = true;
		emit sigQueryTextChanged(this);
	}
}

void KPGSqlQueryResultWindow::slotTextEditCopyAvailable(bool bCopyAvailable)
{
	emit sigCopyAvailable(this, bCopyAvailable);   
}

void KPGSqlQueryResultWindow::slotTextEditUndoAvailable(bool bUndoAvailable)
{
	emit sigUndoAvailable(this, bUndoAvailable);   
}

void KPGSqlQueryResultWindow::slotTextEditRedoAvailable(bool bRedoAvailable)
{
	emit sigRedoAvailable(this, bRedoAvailable);   
}

bool KPGSqlQueryResultWindow::resultEmpty() const
{ 
	return m_pTableResult->numRows() == 0; 
}

KPGDatabase *KPGSqlQueryResultWindow::lookupDatabase(QListViewItem * pItem)
{
	// get selected server and database
	QString strServer(m_pComboBoxServer->currentText());
	QString strDatabase(m_pComboBoxDatabase->currentText());
		
	if(strServer.length() == 0)
	{
		kdError() << "KPGSqlQueryResultWindow::lookupDatabase - server is not selected " << endl;
		return 0;
	}
		
	if(strDatabase.length() == 0)
	{
		kdError() << "KPGSqlQueryResultWindow::lookupDatabase - database is not selected " << endl;
		return 0;
		}
		
	while(pItem)
	{
		if((static_cast <KPGTreeItem *> (pItem))->type() == KPGTreeItem::nodeServer)
		{
		KPGServer *pServer = static_cast <KPGServer *> (pItem);
		if(pServer->text(0) != strServer)
		{ 
			pItem = pItem->nextSibling(); continue; }
					
			//--- traverse list of connected databases
			QListViewItem * pItem2 = pServer->firstChild();
			
			while(pItem2)
			{
			if((static_cast <KPGTreeItem *> (pItem2))->type() == KPGTreeItem::nodeDatabase)
			{
				KPGDatabase *pDatabase = static_cast <KPGDatabase *> (pItem2);
				if(pDatabase->text(0) != strDatabase)
				{ 
					pItem2 = pItem2->nextSibling(); continue; 
				}
			
				if(pDatabase->connection())
				{ 
					return pDatabase;      
				}
				else
				{
					kdError() << "KPGSqlQueryResultWindow::lookupDatabase - database is disconected " << endl;
					return 0;
				}
			}     
			pItem2 = pItem2->nextSibling();
			}  
		}
		pItem = pItem->nextSibling();
	}
	
	return 0;
}

void KPGSqlQueryResultWindow::runQuery(KPGDatabase *pDatabase)
{
	// clear previous result
	m_pTableResult->setNumRows(0);
	m_pTableResult->setNumCols(0);
	m_pTextEditResult->clear();
	m_pTextLabelExecutionTime->setText("");
	m_pqxxResult.clear(); // free memory
		
	int iTransType = m_pComboBoxTransaction->currentItem();
	KPGConnection::eTransType transType = KPGConnection::eTransNormal;
	switch(iTransType)
	{
		case 0: transType = KPGConnection::eTransNormal;
				break;
				
		case 1: transType = KPGConnection::eTransRobust;
				break;
				
		case 2: transType = KPGConnection::eTransNone;
				break;
	}
	
	QString strConectionString( );
	
	m_pTextEditQuery->setEnabled(false);
	m_pComboBoxServer->setEnabled(false);
	m_pComboBoxDatabase->setEnabled(false);
	m_pComboBoxTransaction->setEnabled(false);
	emit sigSetTerminalIcon(this, 1); // red terminal icon
	setCursor(KCursor::waitCursor());
		
	m_ConnectionInThread.disconnectFromServer(); // close connection if any
	m_ConnectionInThread.connectToServer(pDatabase->connection()->options());
	m_ConnectionInThread.setSQL(m_pTextEditQuery->text(), transType, pDatabase->encoderName());
	m_ConnectionInThread.start();
	emit sigQueryResultStateChanged(this);
	emit sigEnableDbFetchActions(false, false);
}

void KPGSqlQueryResultWindow::customEvent( QCustomEvent *pEvent)
{
	PKGQueryResultEvent *pQueryResultEvent = static_cast <PKGQueryResultEvent *> (pEvent);
	
	// display result
	if(pQueryResultEvent->result().size() > 0)
	{
		displayExecutionTime(pQueryResultEvent->executionMiliseconds());   
		displayResult(pQueryResultEvent->result(), pQueryResultEvent->encoderName()); 
		widgetStackResults->raiseWidget(0);
	}
	else
	{
		if(pQueryResultEvent->error().isEmpty() == false)
		{
			m_pTextEditResult->setText(pQueryResultEvent->error());
		}
		else
		{
			displayExecutionTime(pQueryResultEvent->executionMiliseconds());      
			
			if(pQueryResultEvent->connection()->notifications().count() > 0)
			{
				m_pTextEditResult->setText(pQueryResultEvent->connection()->notifications().join(""));
				pQueryResultEvent->connection()->clearNotifications();
			}
			else  
			{
				m_pTextEditResult->setText(i18n("Operation successfull"));
			}
		}
		widgetStackResults->raiseWidget(1); 
	}
	
	m_pTextEditQuery->setEnabled(true);
	m_pComboBoxServer->setEnabled(true);
	m_pComboBoxDatabase->setEnabled(true);
	m_pComboBoxTransaction->setEnabled(true);
	emit sigSetTerminalIcon(this, 2); // blue terminal icon
	setCursor(KCursor::arrowCursor());
	
	m_ConnectionInThread.wait();
	emit sigQueryResultStateChanged(this);
}

void KPGSqlQueryResultWindow::displayExecutionTime(int iMiliseconds)
{
    int hour = 0; int min = 0; int sec = 0;
    int milis = iMiliseconds;
    while(milis >= 1000)
    {
      milis -= 1000;
      sec++;
    }
    while(sec >= 60)
    {
      sec -=60;
      min++;
    }
    while(min >= 60)
    {
      min -= 60;
      hour++;
    }
    
    QString execTime;
    if(hour > 0) execTime.append(QString("%1 h :").arg(hour));
    execTime.append(QString("%1 m : ").arg(min));
    execTime.append(QString("%1 s : ").arg(sec));
    execTime.append(QString("%1 ms").arg(milis));
      
    m_pTextLabelExecutionTime->setText(execTime);
}

// Display SQL result
void KPGSqlQueryResultWindow::displayResult(pqxx::result pqxxResult, const QString& strEncoderName)
{
	m_pqxxResult = pqxxResult;
	
	// Fill output area with query result
	m_nTotalRows = pqxxResult.size();
	m_nTotalCols = pqxxResult.columns();
	m_nFetchedRows = 0;
		
	m_pTableResult->setNumRows(min(m_nTotalRows, NUM_OF_RECORDS_PER_PAGE));
	m_pTableResult->setNumCols(m_nTotalCols);
	
	// field names
	QHeader* hHeader = m_pTableResult->horizontalHeader();
	
	for(unsigned int nCol = 0; nCol < m_nTotalCols; nCol++)
	{
		hHeader->setLabel(nCol, pqxxResult.column_name(nCol));
	}
	
	m_strEncoderName = strEncoderName;
	QTextCodec *pTextCodec = QTextCodec::codecForName(strEncoderName);
	if(!pTextCodec)
		pTextCodec = QTextCodec::codecForLocale(); // hope that correct codec is returned
	
	// data
	for(unsigned int nRow = 0; nRow < m_nTotalRows; nRow++)
	{
		for(unsigned int nCol = 0; nCol < m_nTotalCols; nCol++)
		{
		QString strValue(pTextCodec->toUnicode(pqxxResult[nRow][nCol].c_str()));
		m_pTableResult->setText(nRow, nCol, strValue);
		}
		
		m_nFetchedRows++;
		if(nRow + 1 >= NUM_OF_RECORDS_PER_PAGE)
		{
			break;
		}
	}
	
	for(unsigned int nCol = 0; nCol < m_nTotalCols; nCol++)
	{
		m_pTableResult->adjustColumn(nCol);
	}
	
	pqxxResult.clear(); // free memory
}

void KPGSqlQueryResultWindow::stopQuery()
{
	if(m_ConnectionInThread.running ())
	{
		m_ConnectionInThread.terminate();
		m_ConnectionInThread.wait();
		m_ConnectionInThread.leaveConnection();
		
		m_pTextEditResult->setText(i18n("Operation terminated"));
		widgetStackResults->raiseWidget(1); 
		
		m_pTextEditQuery->setEnabled(true);
		m_pComboBoxServer->setEnabled(true);
		m_pComboBoxDatabase->setEnabled(true);
		m_pComboBoxTransaction->setEnabled(true);
		emit sigSetTerminalIcon(this, 0); // black terminal icon
		m_bQueryTextChanged = false;
		setCursor(KCursor::arrowCursor());
		emit sigQueryResultStateChanged(this);
	} 
}

// Fetch next X rows from large result
void KPGSqlQueryResultWindow::fetchNext()
{
	unsigned int nRows = m_nTotalRows - m_nFetchedRows;
	nRows = min(nRows, NUM_OF_RECORDS_PER_PAGE);
	m_pTableResult->setNumRows(nRows + m_pTableResult->numRows());
	
	QTextCodec *pTextCodec = QTextCodec::codecForName(m_strEncoderName);
	if(!pTextCodec)
		pTextCodec = QTextCodec::codecForLocale(); // hope that correct codec is returned
	
	// data
	unsigned int nRow = m_nFetchedRows;
	unsigned int nFetchedRows = 0;
	for(; nRow < m_nTotalRows; nRow++)
	{
		for(unsigned int nCol = 0; nCol < m_nTotalCols; nCol++)
		{
			QString strValue(pTextCodec->toUnicode(m_pqxxResult[nRow][nCol].c_str()));
			m_pTableResult->setText(nRow, nCol, strValue);
		}
		
		m_nFetchedRows++;
		nFetchedRows++;
		if(nFetchedRows >= NUM_OF_RECORDS_PER_PAGE)
		{
			emit sigEnableDbFetchActions(true, true);
			break;
		}
	}
	
	if(nRow == m_nTotalRows)
	{
		emit sigEnableDbFetchActions(false, false);
		m_pqxxResult.clear(); // free memory
	}
}
  
// Fetch all rows from large result  
void KPGSqlQueryResultWindow::fetchAll()
{
	unsigned int nRows = m_nTotalRows - m_nFetchedRows;
	
	if(nRows > 10000)
	{ 
		if( KMessageBox::questionYesNo(this, QString("There are %1 rows in result. Fething them may take long time and much anmount of memory. Continue ?").arg(nRows), "KPoGre") != KMessageBox::Yes )
		{
			return;
		}
	}
	
	setCursor(KCursor::waitCursor());
	m_pTableResult->setNumRows(nRows + m_pTableResult->numRows());
	
	QTextCodec *pTextCodec = QTextCodec::codecForName(m_strEncoderName);
	if(!pTextCodec)
		pTextCodec = QTextCodec::codecForLocale(); // hope that correct codec is returned
	
	// data
	unsigned int nRow = m_nFetchedRows;
	for(; nRow < m_nTotalRows; nRow++)
	{
		for(unsigned int nCol = 0; nCol < m_nTotalCols; nCol++)
		{
		QString strValue(pTextCodec->toUnicode(m_pqxxResult[nRow][nCol].c_str()));
		m_pTableResult->setText(nRow, nCol, strValue);
		}
		
		m_nFetchedRows++;
	}
	
	setCursor(KCursor::arrowCursor());
	emit sigEnableDbFetchActions(false, false);
	m_pqxxResult.clear(); // free memory
}  
