/***************************************************************************
                          kpgsqlquerresultwindow.h  -  description
                             -------------------
    begin                : So led 17 2004
    copyright            : (C) 2004 by Lumir Vanek
    email                : lvanek@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef KPGSQLQUERRESULTWINDOW_H
#define KPGSQLQUERRESULTWINDOW_H

#include "kpgsqlqueryresultwindowbase.h"

#include "kpgconnectioninthread.h"

class QListViewItem;
class KTextEdit;
class KPGSyntaxHighlighter;
class KPGDatabase;

/**Window that allow edit and run SQL query, and display result
  *@author Lumir Vanek
  */

class KPGSqlQueryResultWindow : public KPGSqlQueryResultWindowBase
{
   Q_OBJECT
public: 
	KPGSqlQueryResultWindow(QWidget *parent=0, const char *name=0);
	~KPGSqlQueryResultWindow();

  	KTextEdit * editQuery() const { return m_pTextEditQuery; }
  	
  	QTable *tableResult() const { return m_pTableResult; }

	// Is SQL query changed after last open/save ?
	bool queryTextChanged() const { return m_bQueryTextChanged; }
	
	// Set flag indicating changed query text
	void setQueryTextChanged(bool changed) { m_bQueryTextChanged = changed; }
	
	// Signalize, if result contain at least one row
	bool resultEmpty() const;
  
  	bool fetchedAllRows() const { return m_nTotalRows == m_nFetchedRows;  }
  
  	// Lookup KPGDatabase for selected Server and database in combo boxes 
  	KPGDatabase * lookupDatabase(QListViewItem *);
  
	// Run SQL query
	void runQuery(KPGDatabase *);
	
	// Terminate thread that run SQL query
	void stopQuery();
	
	// Returns TRUE if the connection thread is running; otherwise returns FALSE.
	bool running() const { return  m_ConnectionInThread.running(); }
	
	// Fetch next X rows from large result
	void fetchNext();
	
	// Fetch all rows from large result  
	void fetchAll();
				
protected:

    // Receive event from KPGConnectionInThread
    virtual void customEvent( QCustomEvent * );

    // Display SQL result
    void displayResult(pqxx::result, const QString &);  
    
    // Display query execution time
    void displayExecutionTime(int);
  
protected slots:
    
	// Changes syntax highlighting colors.
	void slotSqlEditorSettingsChanged();
  
	void slotServerChanged(const QString &);

	// Called, when user change text in SQL input area. Mark document as modyfied.
   	void slotTextEditQueryTextChanged();
	
	// Called, when user copu action availability is changed in SQL input area
	void slotTextEditCopyAvailable(bool);

	// Called, when undo action availability is changed in SQL input area
	void slotTextEditUndoAvailable(bool);

	// Called, when redo action availability is changed in SQL input area
	void slotTextEditRedoAvailable(bool);

private:
	int max(int a, int b) { return a > b ? a : b; }
	int min(int a, int b) { return a < b ? a : b; }
		
signals:

	// Fired, when info about databases list is needed
	void sigRequestDatabasesList(KPGSqlQueryResultWindow *, const QString&);
		
	// Fired when availability of text for cut/copy is changed
	void sigCopyAvailable(KPGSqlQueryResultWindow *, bool);

	// Fired when availability of undo is changed
	void sigUndoAvailable(KPGSqlQueryResultWindow *, bool);
		
	// Fired when availability of redo is changed
	void sigRedoAvailable(KPGSqlQueryResultWindow *, bool);
				
	// Fired when user change text in query edit widget
	void sigQueryTextChanged(KPGSqlQueryResultWindow *);
    
	// Fired for request to change terminal icon in QTabWidget 
	void sigSetTerminalIcon(KPGSqlQueryResultWindow *, int);
		
	// Called when SQL query started or finished
	void sigQueryResultStateChanged(KPGSqlQueryResultWindow *);
		
	// Fired for enabling/disabling file menu actions
	void sigEnableFileActions(bool, bool, bool);
		
	// Enables/disables actions Database -> Fetch Next, Fetch All
    void sigEnableDbFetchActions(bool, bool);
  
protected:

	// Syntax highlighter for m_pMultiLineEditQuery
	KPGSyntaxHighlighter *m_pSyntaxHighlighter;
	
	bool m_bQueryTextChanged;
	
	// Thread that run queries on background
	KPGConnectionInThread m_ConnectionInThread;
	
	// Encoder name for encoding strings
	QString m_strEncoderName;
	
	// Resultset to display
	pqxx::result m_pqxxResult;
	
	unsigned int m_nTotalRows; // Number of rows in result
	unsigned int m_nFetchedRows; // Number of fetched rows from result
	unsigned int m_nTotalCols; // Number oc columns in result
};

#endif
