//
// C++ Implementation: kpgsqlqueryresultchildview
//
// Description: 
//
//
// Author: Lumir Vanek <lvanek@users.sourceforge.net>, (C) 2006
//
// Copyright: See COPYING file that comes with this distribution
//
//
#include "kpgsqlqueryresultchildview.h"

// include files for Qt
#include <qdir.h>
#include <qlayout.h>

// include files for KDE
#include <klocale.h>
#include <kmessagebox.h>
#include <kio/job.h>
#include <kio/netaccess.h>
#include <kiconloader.h>
#include <ktextedit.h>

#include "kpgsqlquerresultwindow.h"
#include "kpogreview.h"

KPGSqlQueryResultChildView::KPGSqlQueryResultChildView(KPoGreView *pPoGreView)
 : KMdiChildView(i18n("SQL"), pPoGreView, "KPGSqlQueryResultChildView")
{
	m_pPoGreView = pPoGreView;
	
	QBoxLayout* pLayout = new QHBoxLayout(this, 0, -1, "layout" );
	m_pSqlQueryResultWindow = new KPGSqlQueryResultWindow(this, "SQL Query/result window");
	pLayout->addWidget( m_pSqlQueryResultWindow );
	
	setIcon( SmallIcon("openterm") );
	newDocument();
}

KPGSqlQueryResultChildView::~KPGSqlQueryResultChildView()
{
}

const KURL& KPGSqlQueryResultChildView::URL() const
{
  return m_urlSqlFile;
}

bool KPGSqlQueryResultChildView::newDocument()
{
  /////////////////////////////////////////////////
  // TODO: Add your document initialization code here
  /////////////////////////////////////////////////
  m_pSqlQueryResultWindow->setQueryTextChanged(false);
  m_urlSqlFile.setFileName(i18n("Untitled"));

  return true;
}

bool KPGSqlQueryResultChildView::openDocument(const KURL& url, const char */*format*/ /*=0*/)
{
	QString tmpfile;
	KIO::NetAccess::download( url, tmpfile, m_pSqlQueryResultWindow);
	
	QFile sqlfile(tmpfile);
	if( !sqlfile.open(IO_ReadOnly) )
	{ 
		QString error =  i18n("Error opening ");
		error.append(tmpfile);
		
		KMessageBox::error(this, error, "KPoGre");
		KIO::NetAccess::removeTempFile( tmpfile );
		return false;
	}
	else
	{ 
		QTextStream inStream( &sqlfile );
		while( !inStream.atEnd() )
		{ 
			m_pSqlQueryResultWindow->editQuery()->setText( inStream.read() );
		}
		sqlfile.close();
	}
	
	KIO::NetAccess::removeTempFile( tmpfile );
	m_pSqlQueryResultWindow->setQueryTextChanged(false);
	m_urlSqlFile = url;
	return true;
}

bool KPGSqlQueryResultChildView::saveDocument(const KURL& url, const char */*format*/ /*=0*/)
{
	//if(m_urlSqlFile.path().isEmpty())
	//	return true;

	QFile file(url.path());
	if(file.open(IO_WriteOnly) == true)
	{ 
		QString strSQL = m_pSqlQueryResultWindow->editQuery()->text();

 		file.writeBlock(strSQL, strSQL.length());
		file.flush();
		file.close();
	}
	else
	{ 
		KMessageBox::error(this,
						 i18n("Write error. Can't create file %1").arg(url.path()),
						 "KPoGre");
	}

  	m_pSqlQueryResultWindow->setQueryTextChanged(false);
	m_urlSqlFile = url;
  	return true;
}

const bool KPGSqlQueryResultChildView::isModified() const 
{
	return m_pSqlQueryResultWindow->queryTextChanged();
}

bool KPGSqlQueryResultChildView::requestClose()
{
	if(m_pSqlQueryResultWindow->running())
	{
		KMessageBox::sorry(this, i18n("Query is running, cannot close !"), "KPoGre");
						 
		return false;
	}
	
	if(isModified())
	{
		if(KMessageBox::questionYesNo(this, i18n("SQL script is not saved, really close window ?"), "KPoGre") == KMessageBox::Yes) 
		{
			return true;
		}
		else
		{
			return false;
		}
	}
	
	return true;
}
